{% load l10n static %}
<script type="text/javascript"
        id="django-admin-prepopulated-fields-constants"
        class="override-prepopulated_fields_js"
        src="{% static "admin/js/prepopulate_init.js" %}"
        data-prepopulated-fields="{{ prepopulated_fields_json }}">
</script>
